@@ -31,17 +31,17 @@ from oslotest import base as test_base
 from oslo_serialization import jsonutils
 
 
-class ReprObject(object):
+class ReprObject:
     def __repr__(self):
         return 'repr'
 
 
-class JSONUtilsTestMixin(object):
+class JSONUtilsTestMixin:
 
     json_impl = None
 
     def setUp(self):
-        super(JSONUtilsTestMixin, self).setUp()
+        super().setUp()
         self.json_patcher = mock.patch.multiple(
             jsonutils, json=self.json_impl,
         )
@@ -49,7 +49,7 @@ class JSONUtilsTestMixin(object):
 
     def tearDown(self):
         self.json_patcher.stop()
-        super(JSONUtilsTestMixin, self).tearDown()
+        super().tearDown()
 
     def test_dumps(self):
         self.assertEqual('{"a": "b"}', jsonutils.dumps({'a': 'b'}))
@@ -89,24 +89,24 @@ class JSONUtilsTestMixin(object):
 
     def test_loads_unicode(self):
         self.assertIsInstance(jsonutils.loads(b'"foo"'), str)
-        self.assertIsInstance(jsonutils.loads(u'"foo"'), str)
+        self.assertIsInstance(jsonutils.loads('"foo"'), str)
 
         # 'test' in Ukrainian
-        i18n_str_unicode = u'"\u0442\u0435\u0441\u0442"'
+        i18n_str_unicode = '"\u0442\u0435\u0441\u0442"'
         self.assertIsInstance(jsonutils.loads(i18n_str_unicode), str)
 
         i18n_str = i18n_str_unicode.encode('utf-8')
         self.assertIsInstance(jsonutils.loads(i18n_str), str)
 
     def test_loads_with_kwargs(self):
-        jsontext = u'{"foo": 3}'
+        jsontext = '{"foo": 3}'
         result = jsonutils.loads(jsontext, parse_int=lambda x: 5)
         self.assertEqual(5, result['foo'])
 
     def test_load(self):
 
-        jsontext = u'{"a": "\u0442\u044d\u0441\u0442"}'
-        expected = {u'a': u'\u0442\u044d\u0441\u0442'}
+        jsontext = '{"a": "\u0442\u044d\u0441\u0442"}'
+        expected = {'a': '\u0442\u044d\u0441\u0442'}
 
         for encoding in ('utf-8', 'cp1251'):
             fp = io.BytesIO(jsontext.encode(encoding))
@@ -128,7 +128,7 @@ class JSONUtilsTestJson(JSONUtilsTestMixin, test_base.BaseTestCase):
 
 class ToPrimitiveTestCase(test_base.BaseTestCase):
     def setUp(self):
-        super(ToPrimitiveTestCase, self).setUp()
+        super().setUp()
         self.trans_fixture = self.useFixture(fixture.Translation())
 
     def test_bytes(self):
@@ -159,6 +159,15 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
         x = datetime.datetime(1920, 2, 3, 4, 5, 6, 7)
         self.assertEqual(x, jsonutils.to_primitive(x, convert_datetime=False))
 
+    def test_date(self):
+        x = datetime.date(1920, 2, 3)
+        self.assertEqual('1920-02-03',
+                         jsonutils.to_primitive(x))
+
+    def test_date_preserve(self):
+        x = datetime.date(1920, 2, 3)
+        self.assertEqual(x, jsonutils.to_primitive(x, convert_datetime=False))
+
     def test_DateTime(self):
         x = xmlrpclib.DateTime()
         x.decode("19710203T04:05:06")
@@ -166,7 +175,7 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
                          jsonutils.to_primitive(x))
 
     def test_iter(self):
-        class IterClass(object):
+        class IterClass:
             def __init__(self):
                 self.data = [1, 2, 3, 4, 5]
                 self.index = 0
@@ -185,7 +194,7 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
         self.assertEqual([1, 2, 3, 4, 5], jsonutils.to_primitive(x))
 
     def test_iteritems(self):
-        class IterItemsClass(object):
+        class IterItemsClass:
             def __init__(self):
                 self.data = dict(a=1, b=2, c=3).items()
                 self.index = 0
@@ -198,7 +207,7 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
         self.assertEqual({'a': 1, 'b': 2, 'c': 3}, p)
 
     def test_iteritems_with_cycle(self):
-        class IterItemsClass(object):
+        class IterItemsClass:
             def __init__(self):
                 self.data = dict(a=1, b=2, c=3)
                 self.index = 0
@@ -217,7 +226,7 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
 
     def test_items(self):
         # Use items() when iteritems() is not available.
-        class ItemsClass(object):
+        class ItemsClass:
             def __init__(self):
                 self.data = dict(a=1, b=2, c=3)
 
@@ -229,7 +238,7 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
         self.assertEqual({'a': 1, 'b': 2, 'c': 3}, p)
 
     def test_precedence_items_iteritems(self):
-        class ItemsIterItemsClass(object):
+        class ItemsIterItemsClass:
             def items(self):
                 return {'items': 'items'}
 
@@ -261,9 +270,8 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
         p = jsonutils.to_primitive(x)
         self.assertEqual({'a': 1, 'b': 2, 'c': 3}, p)
 
-    @mock.patch('warnings.warn')
-    def test_instance(self, warn_mock):
-        class MysteryClass(object):
+    def test_instance(self):
+        class MysteryClass:
             a = 10
 
             def __init__(self):
@@ -273,12 +281,11 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
         self.assertEqual(dict(b=1),
                          jsonutils.to_primitive(x, convert_instances=True))
 
-        self.assertEqual(x, jsonutils.to_primitive(x))
-        warn_mock.assert_called_once()
+        self.assertRaises(ValueError, jsonutils.to_primitive, x)
 
     def test_typeerror(self):
         x = bytearray  # Class, not instance
-        self.assertEqual(u"<class 'bytearray'>", jsonutils.to_primitive(x))
+        self.assertEqual("<class 'bytearray'>", jsonutils.to_primitive(x))
 
     def test_nasties(self):
         def foo():
@@ -286,15 +293,15 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
         x = [datetime, foo, dir]
         ret = jsonutils.to_primitive(x)
         self.assertEqual(3, len(ret))
-        self.assertTrue(ret[0].startswith(u"<module 'datetime' from ") or
-                        ret[0].startswith(u"<module 'datetime' (built-in)"))
+        self.assertTrue(ret[0].startswith("<module 'datetime' from ") or
+                        ret[0].startswith("<module 'datetime' (built-in)"))
         self.assertTrue(ret[1].startswith(
             '<function ToPrimitiveTestCase.test_nasties.<locals>.foo at 0x'
         ))
         self.assertEqual('<built-in function dir>', ret[2])
 
     def test_depth(self):
-        class LevelsGenerator(object):
+        class LevelsGenerator:
             def __init__(self, levels):
                 self._levels = levels
 
@@ -325,12 +332,12 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
         self.assertEqual({'ip_addr': '1.2.3.4'}, ret)
 
     def test_ipaddr_using_ipaddress_v4(self):
-        thing = {'ip_addr': ipaddress.ip_address(u'192.168.0.1')}
+        thing = {'ip_addr': ipaddress.ip_address('192.168.0.1')}
         ret = jsonutils.to_primitive(thing)
         self.assertEqual({'ip_addr': '192.168.0.1'}, ret)
 
     def test_ipaddr_using_ipaddress_v6(self):
-        thing = {'ip_addr': ipaddress.ip_address(u'2001:db8::')}
+        thing = {'ip_addr': ipaddress.ip_address('2001:db8::')}
         ret = jsonutils.to_primitive(thing)
         self.assertEqual({'ip_addr': '2001:db8::'}, ret)
 
@@ -354,8 +361,7 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
     def test_fallback(self):
         obj = ReprObject()
 
-        ret = jsonutils.to_primitive(obj)
-        self.assertIs(obj, ret)
+        self.assertRaises(ValueError, jsonutils.to_primitive, obj)
 
         ret = jsonutils.to_primitive(obj, fallback=repr)
         self.assertEqual('repr', ret)
@@ -364,8 +370,7 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
         obj = ReprObject()
         obj_list = [obj]
 
-        ret = jsonutils.to_primitive(obj_list)
-        self.assertEqual([obj], ret)
+        self.assertRaises(ValueError, jsonutils.to_primitive, obj_list)
 
         ret = jsonutils.to_primitive(obj_list, fallback=repr)
         self.assertEqual(['repr'], ret)
@@ -390,7 +395,7 @@ class ToPrimitiveTestCase(test_base.BaseTestCase):
         self.assertEqual("type:int", ret)
 
     def test_fallback_typeerror(self):
-        class NotIterable(object):
+        class NotIterable:
             # __iter__ is not callable, cause a TypeError in to_primitive()
             __iter__ = None
 
